import 'package:flutter/widgets.dart';
import 'package:flutterskel/configuration/Configuration.dart' as arch;
import 'package:flutterskel/resources/strings/StringResources.dart' as arch;
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart' as arch;
import 'package:flutterskel/routing/page/PageRouter.dart' as arch;
import 'package:flutterskel/routing/url/UrlRouter.dart' as arch;
import 'package:flutterskel/userinformation/UserInformationRepository.dart' as arch;
import 'package:provider/provider.dart';

abstract class ProviderBuilder<
                  // Data
                  DataSource,
                  // Routing
                  UrlRouter extends arch.UrlRouter,
                  PageRouter extends arch.PageRouter,
                  // Configuration
                  Configuration extends arch.Configuration,
                  // User Information
                  UserInformationRepository extends arch.UserInformationRepository,
                  // Theme Related: Repository -> Theme -> Fonts -> Dimensions/Colors
                  ThemeRepository extends arch.ThemeRepository,
                  // Strings
                  Strings extends arch.StringResources,
                  // Images & Icons
                  ImageResources,
                  IconsClass> {

  // DataSource
  DataSource _dataSource;
  DataSource buildDataSource();
  DataSource get dataSource => _dataSource ??= buildDataSource();

  // Page Router
  PageRouter _pageRouter;
  PageRouter buildPageRouter();
  PageRouter get pageRouter => _pageRouter ??= buildPageRouter();

  // Url Router
  UrlRouter _urlRouter;
  UrlRouter buildUrlRouter();
  UrlRouter get urlRouter => _urlRouter ??= buildUrlRouter();

  // Configuration
  Configuration _configuration;
  Configuration buildConfiguration();
  Configuration get configuration => _configuration ??= buildConfiguration();

  // Images
  ImageResources _images;
  ImageResources buildImages();
  ImageResources get images => _images ??= buildImages();

  // Strings
  StringsRepository _stringsRepository;
  Strings buildStringsResources();
  StringsRepository get stringsRepository => _stringsRepository ??= StringsRepository<Strings>(
    strings: buildStringsResources()
  );

  // User Information
  UserInformationRepository _userInformationRepository;
  UserInformationRepository buildUserInformationRepository();
  UserInformationRepository get userInformationRepository => _userInformationRepository ??= buildUserInformationRepository();

  // Icons
  IconsClass _icons;
  IconsClass buildIcons();
  IconsClass get icons => _icons ??= buildIcons();

  // Theme
  ThemeRepository _themeRepository;
  ThemeRepository buildThemeRepository();
  ThemeRepository get themeRepository => _themeRepository ??= buildThemeRepository();


  List<SingleChildCloneableWidget> get providers => [
    // DataSource
    Provider<DataSource>.value(value: dataSource),

    // Routing
    Provider<UrlRouter>.value(value: urlRouter),
    Provider<PageRouter>.value(value: pageRouter),

    // Resources
    Provider<ImageResources>.value(value: images),
    Provider<IconsClass>.value(value: icons),

    // Change Notifiers
    ChangeNotifierProvider<Configuration>.value(value: configuration),
    ChangeNotifierProvider<UserInformationRepository>.value(value: userInformationRepository),
    ChangeNotifierProvider<StringsRepository>.value(value: stringsRepository),
    ChangeNotifierProvider<ThemeRepository>.value(value: themeRepository)
  ];

  MultiProvider build({@required Widget child}) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}
