import 'package:flutter/widgets.dart';

/// Abstract class for url-based routing
/// Responsibilities:
/// Identify the appropriate page router function to call
/// Identify and handle urls that are not handled by the app
abstract class UrlRouter {

  /// Main function for this app
  /// [url] The url to navigate to
  /// [bootstrapInformation] Information that is used to bootstrap the next page, (image urls, titles etc.)
  /// [heroTags] Hero tags for widgets
  void loadUrl({
    @required BuildContext context,
    @required String url,
    Map<String, String> bootstrapInformation,
    Map<String, String> heroTags
  });

  /// Helper function to convert a url to a widget
  /// Can be used directly from screens
  /// Can also be utilised by [loadUrl]
  /// [url] The url to navigate to
  /// [bootstrapInformation] Information that is used to bootstrap the next page, (image urls, titles etc.)
  /// [heroTags] Hero tags for widgets
  Widget getScreenWidget({
    BuildContext context,
    String url,
    Map<String, String> bootstrapInformation,
    Map<String, String> heroTags
  });

}