import 'package:flutter/widgets.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart' as arch;
import 'package:flutterskel/resources/dimensions/DimensionResources.dart' as arch;
import 'package:flutterskel/resources/fonts/FontResources.dart' as arch;
import 'package:flutterskel/resources/strings/StringResources.dart' as arch;
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart' as arch;
import 'package:flutterskel/resources/themes/ThemeResources.dart' as arch;
import 'package:flutterskel/routing/page/PageRouter.dart' as arch;
import 'package:flutterskel/routing/url/UrlRouter.dart' as arch;
import 'package:flutterskel/userinformation/UserInformationRepository.dart' as arch;
import 'package:flutterskel/configuration/Configuration.dart' as arch;
import 'package:provider/provider.dart';

/// Raw abstract State class
/// This class should only be extended by an abstract class within the app, which fills in all the info except [StateWidget]
/// [StateWidget] should be filled in by the final StatefulWidget class
abstract class RawBaseStatelessWidget<
                  // Data
                  DataSource,
                  // Routing
                  UrlRouter extends arch.UrlRouter,
                  PageRouter extends arch.PageRouter,
                  // Configuration
                  Configuration extends arch.Configuration,
                  // User Information
                  UserInformationRepository extends arch.UserInformationRepository,
                  // Themes (Fonts/Dimensions/Colors)
                  ThemeRepository extends arch.ThemeRepository,
                  Theme extends arch.ThemeResources,
                  Colors extends arch.ColorResources,
                  Dimensions extends arch.DimensionResources,
                  Fonts extends arch.FontResources,
                  // Strings
                  Strings extends arch.StringResources,
                  // Images and Icons
                  Images,
                  Icons> extends StatelessWidget {

  BuildContext _context;

  RawBaseStatelessWidget({
    Key key
  }) : super(key: key);

  @override
  @mustCallSuper
  Widget build(BuildContext context) {
    _context = context;
    return null;
  }
  
  // region Providers
  // Data
  DataSource get dataSource => Provider.of<DataSource>(_context);

  // Routing
  UrlRouter get urlRouter => Provider.of<UrlRouter>(_context);
  PageRouter get pageRouter => Provider.of<PageRouter>(_context);

  // Configuration
  Configuration get configuration => Provider.of<Configuration>(_context);

  // User Information
  UserInformationRepository get userInformation => Provider.of<UserInformationRepository>(_context);

  // Strings
  StringsRepository get stringsRepository => Provider.of<StringsRepository>(_context);
  Strings get strings => stringsRepository.strings;

  // Theme
  ThemeRepository get themeRepo => Provider.of<ThemeRepository>(_context);
  Theme get theme => themeRepo.theme;
  Colors get colors => theme.colors;
  Dimensions get dimensions => theme.dimensions;
  Fonts get fonts => theme.fonts;

  // Images and Icons
  Images get images => Provider.of<Images>(_context);
  Icons get icons => Provider.of<Icons>(_context);
  // endregion Providers

}
